@@ -15,8 +15,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #include <string.h>
@@ -29,11 +29,26 @@
 GST_DEBUG_CATEGORY_STATIC (gst_jack_audio_client_debug);
 #define GST_CAT_DEFAULT gst_jack_audio_client_debug
 
+static void
+jack_log_error (const gchar * msg)
+{
+  GST_ERROR ("%s", msg);
+}
+
+static void
+jack_info_error (const gchar * msg)
+{
+  GST_INFO ("%s", msg);
+}
+
 void
 gst_jack_audio_client_init (void)
 {
   GST_DEBUG_CATEGORY_INIT (gst_jack_audio_client_debug, "jackclient", 0,
       "jackclient helpers");
+
+  jack_set_error_function (jack_log_error);
+  jack_set_info_function (jack_info_error);
 }
 
 /* a list of global connections indexed by id and server. */
