@@ -13,8 +13,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #ifndef _GST_SOUP_HTTP_CLIENT_SINK_H_
@@ -66,7 +66,7 @@ struct _GstSoupHttpClientSink
   char *user_agent;
   gboolean automatic_redirect;
   gchar **cookies;
-
+  SoupLoggerLogLevel log_level;
 };
 
 struct _GstSoupHttpClientSinkClass
