@@ -22,11 +22,10 @@
 
 /**
  * SECTION:element-progressreport
- * @title: progressreport
  *
  * The progressreport element can be put into a pipeline to report progress,
  * which is done by doing upstream duration and position queries in regular
- * (real-time) intervals. Both the interval and the preferred query format
+ * (real-time) intervals. Both the interval and the prefered query format
  * can be specified via the #GstProgressReport:update-freq and the
  * #GstProgressReport:format property.
  *
@@ -54,14 +53,15 @@
  * is in reference to an internal point of a pipeline and not the pipeline as
  * a whole).
  *
- * ## Example launch line
+ * <refsect2>
+ * <title>Example launch line</title>
  * |[
  * gst-launch-1.0 -m filesrc location=foo.ogg ! decodebin ! progressreport update-freq=1 ! audioconvert ! audioresample ! autoaudiosink
  * ]| This shows a progress query where a duration is available.
  * |[
  * gst-launch-1.0 -m audiotestsrc ! progressreport update-freq=1 ! audioconvert ! autoaudiosink
  * ]| This shows a progress query where no duration is available.
- *
+ * </refsect2>
  */
 
 #ifdef HAVE_CONFIG_H
@@ -165,10 +165,10 @@ gst_progress_report_class_init (GstProgressReportClass * g_class)
           "Format to use for the querying", DEFAULT_FORMAT,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
-  gst_element_class_add_static_pad_template (element_class,
-      &progress_report_sink_template);
-  gst_element_class_add_static_pad_template (element_class,
-      &progress_report_src_template);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&progress_report_sink_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&progress_report_src_template));
 
   gst_element_class_set_static_metadata (element_class, "Progress report",
       "Testing",
@@ -252,9 +252,6 @@ gst_progress_report_do_query (GstProgressReport * filter, GstFormat format,
       cur = gst_segment_to_stream_time (&base->segment, format,
           GST_BUFFER_TIMESTAMP (buf));
       total = base->segment.duration;
-    } else if (format == GST_FORMAT_BUFFERS) {
-      cur = filter->buffer_count;
-      total = -1;
     } else {
       return FALSE;
     }
@@ -325,7 +322,7 @@ gst_progress_report_do_query (GstProgressReport * filter, GstFormat format,
 }
 
 static void
-gst_progress_report_report (GstProgressReport * filter, gint64 cur_time_s,
+gst_progress_report_report (GstProgressReport * filter, GTimeVal cur_time,
     GstBuffer * buf)
 {
   GstFormat try_formats[] = { GST_FORMAT_TIME, GST_FORMAT_BYTES,
@@ -338,7 +335,7 @@ gst_progress_report_report (GstProgressReport * filter, gint64 cur_time_s,
   glong run_time;
   gint hh, mm, ss;
 
-  run_time = cur_time_s - filter->start_time_s;
+  run_time = cur_time.tv_sec - filter->start_time.tv_sec;
 
   hh = (run_time / 3600) % 100;
   mm = (run_time / 60) % 60;
@@ -387,9 +384,10 @@ gst_progress_report_sink_event (GstBaseTransform * trans, GstEvent * event)
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_EOS:
     {
-      gint64 cur_time_s = g_get_real_time () / G_USEC_PER_SEC;
+      GTimeVal cur_time;
 
-      gst_progress_report_report (filter, cur_time_s, NULL);
+      g_get_current_time (&cur_time);
+      gst_progress_report_report (filter, cur_time, NULL);
       break;
     }
     default:
@@ -403,22 +401,22 @@ gst_progress_report_transform_ip (GstBaseTransform * trans, GstBuffer * buf)
 {
   GstProgressReport *filter;
   gboolean need_update;
-  gint64 cur_time;
+  GTimeVal cur_time;
 
-  cur_time = g_get_real_time () / G_USEC_PER_SEC;
+  g_get_current_time (&cur_time);
 
   filter = GST_PROGRESS_REPORT (trans);
 
   /* Check if update_freq seconds have passed since the last update */
   GST_OBJECT_LOCK (filter);
-  need_update = (cur_time - filter->last_report_s) >= filter->update_freq;
-  filter->buffer_count++;
+  need_update =
+      ((cur_time.tv_sec - filter->last_report.tv_sec) >= filter->update_freq);
   GST_OBJECT_UNLOCK (filter);
 
   if (need_update) {
     gst_progress_report_report (filter, cur_time, buf);
     GST_OBJECT_LOCK (filter);
-    filter->last_report_s = cur_time;
+    filter->last_report = cur_time;
     GST_OBJECT_UNLOCK (filter);
   }
 
@@ -432,9 +430,8 @@ gst_progress_report_start (GstBaseTransform * trans)
 
   filter = GST_PROGRESS_REPORT (trans);
 
-  filter->start_time_s = filter->last_report_s =
-      g_get_real_time () / G_USEC_PER_SEC;
-  filter->buffer_count = 0;
+  g_get_current_time (&filter->last_report);
+  filter->start_time = filter->last_report;
 
   return TRUE;
 }
