@@ -52,7 +52,7 @@ gst_ebml_write_class_init (GstEbmlWriteClass * klass)
 static void
 gst_ebml_write_init (GstEbmlWrite * ebml)
 {
-  ebml->agg = NULL;
+  ebml->srcpad = NULL;
   ebml->pos = 0;
   ebml->last_pos = G_MAXUINT64; /* force segment event */
 
@@ -68,7 +68,7 @@ gst_ebml_write_finalize (GObject * object)
 {
   GstEbmlWrite *ebml = GST_EBML_WRITE (object);
 
-  ebml->agg = NULL;
+  gst_object_unref (ebml->srcpad);
 
   if (ebml->cache) {
     gst_byte_writer_free (ebml->cache);
@@ -98,12 +98,12 @@ gst_ebml_write_finalize (GObject * object)
  * Returns: a new #GstEbmlWrite
  */
 GstEbmlWrite *
-gst_ebml_write_new (GstAggregator * agg)
+gst_ebml_write_new (GstPad * srcpad)
 {
   GstEbmlWrite *ebml =
       GST_EBML_WRITE (g_object_new (GST_TYPE_EBML_WRITE, NULL));
 
-  ebml->agg = agg;
+  ebml->srcpad = gst_object_ref (srcpad);
   ebml->timestamp = GST_CLOCK_TIME_NONE;
 
   gst_ebml_write_reset (ebml);
@@ -207,20 +207,25 @@ gst_ebml_write_set_cache (GstEbmlWrite * ebml, guint size)
   ebml->cache_pos = ebml->pos;
 }
 
-static void
+static gboolean
 gst_ebml_writer_send_segment_event (GstEbmlWrite * ebml, guint64 new_pos)
 {
   GstSegment segment;
+  gboolean res;
 
   GST_INFO ("seeking to %" G_GUINT64_FORMAT, new_pos);
 
-  gst_segment_init (&segment,
-      ebml->streamable ? GST_FORMAT_TIME : GST_FORMAT_BYTES);
+  gst_segment_init (&segment, GST_FORMAT_BYTES);
   segment.start = new_pos;
   segment.stop = -1;
   segment.position = 0;
 
-  gst_aggregator_update_segment (ebml->agg, &segment);
+  res = gst_pad_push_event (ebml->srcpad, gst_event_new_segment (&segment));
+
+  if (!res)
+    GST_WARNING ("seek to %" G_GUINT64_FORMAT "failed", new_pos);
+
+  return res;
 }
 
 /**
@@ -262,7 +267,7 @@ gst_ebml_write_flush_cache (GstEbmlWrite * ebml, gboolean is_keyframe,
       GST_BUFFER_FLAG_SET (buffer, GST_BUFFER_FLAG_DELTA_UNIT);
     }
     ebml->last_pos = ebml->pos;
-    ebml->last_write_result = gst_aggregator_finish_buffer (ebml->agg, buffer);
+    ebml->last_write_result = gst_pad_push (ebml->srcpad, buffer);
   } else {
     gst_buffer_unref (buffer);
   }
@@ -302,7 +307,7 @@ gst_ebml_write_element_new (GstEbmlWrite * ebml, GstMapInfo * map, guint size)
  * gst_ebml_write_element_id:
  * @data_inout: Pointer to data pointer
  * @id: Element ID that should be written.
- *
+ * 
  * Write element ID into a buffer.
  */
 static void
@@ -401,7 +406,7 @@ gst_ebml_write_element_data (guint8 ** data_inout, guint8 * write,
  * @buf_data: Start of data to push from @buf (or NULL for whole buffer).
  * @buf_data_end: Data pointer positioned after the last byte in @buf_data (or
  * NULL for whole buffer).
- *
+ * 
  * Write out buffer by moving it to the next element.
  */
 static void
@@ -468,7 +473,7 @@ gst_ebml_write_element_push (GstEbmlWrite * ebml, GstBuffer * buf,
       GST_BUFFER_FLAG_UNSET (buf, GST_BUFFER_FLAG_DISCONT);
     }
     ebml->last_pos = ebml->pos;
-    ebml->last_write_result = gst_aggregator_finish_buffer (ebml->agg, buf);
+    ebml->last_write_result = gst_pad_push (ebml->srcpad, buf);
   } else {
     gst_buffer_unref (buf);
   }
@@ -479,7 +484,7 @@ gst_ebml_write_element_push (GstEbmlWrite * ebml, GstBuffer * buf,
  * gst_ebml_write_seek:
  * @ebml: #GstEbmlWrite
  * @pos: Seek position.
- *
+ * 
  * Seek.
  */
 void
@@ -522,7 +527,7 @@ gst_ebml_write_seek (GstEbmlWrite * ebml, guint64 pos)
 /**
  * gst_ebml_write_get_uint_size:
  * @num: Number to be encoded.
- *
+ * 
  * Get number of bytes needed to write a uint.
  *
  * Returns: Encoded uint length.
@@ -715,14 +720,14 @@ gst_ebml_write_utf8 (GstEbmlWrite * ebml, guint32 id, const gchar * str)
  * gst_ebml_write_date:
  * @ebml: #GstEbmlWrite
  * @id: Element ID.
- * @date: Date in nanoseconds since the unix epoch.
+ * @date: Date in seconds since the unix epoch.
  *
  * Write date element.
  */
 void
 gst_ebml_write_date (GstEbmlWrite * ebml, guint32 id, gint64 date)
 {
-  gst_ebml_write_sint (ebml, id, date - GST_EBML_DATE_OFFSET);
+  gst_ebml_write_sint (ebml, id, (date - GST_EBML_DATE_OFFSET) * GST_SECOND);
 }
 
 /**
@@ -734,7 +739,7 @@ gst_ebml_write_date (GstEbmlWrite * ebml, guint32 id, gint64 date)
  *
  * Master writing is annoying. We use a size marker of
  * the max. allowed length, so that we can later fill it
- * in validly.
+ * in validly. 
  *
  * Returns: Master starting position.
  */
@@ -827,9 +832,9 @@ gst_ebml_write_binary (GstEbmlWrite * ebml,
  * @ebml: #GstEbmlWrite
  * @id: Element ID.
  * @length: Length of the data
- *
+ * 
  * Write header of the binary element (use with gst_ebml_write_buffer function).
- *
+ * 
  * For things like video frames and audio samples,
  * you want to use this function, as it doesn't have
  * the overhead of memcpy() that other functions
@@ -857,7 +862,7 @@ gst_ebml_write_buffer_header (GstEbmlWrite * ebml, guint32 id, guint64 length)
 /**
  * gst_ebml_write_buffer:
  * @ebml: #GstEbmlWrite
- * @buf: #GstBuffer containing the data.
+ * @buf: #GstBuffer cointaining the data.
  *
  * Write  binary element (see gst_ebml_write_buffer_header).
  */
@@ -875,7 +880,7 @@ gst_ebml_write_buffer (GstEbmlWrite * ebml, GstBuffer * buf)
  * @num: New value.
  *
  * Replace uint with a new value.
- *
+ * 
  * When replacing a uint, we assume that it is *always*
  * 8-byte, since that's the safest guess we can do. This
  * is just for simplicity.
@@ -906,12 +911,12 @@ gst_ebml_replace_uint (GstEbmlWrite * ebml, guint64 pos, guint64 num)
  * @ebml: #GstEbmlWrite
  * @doctype: Document type.
  * @version: Document type version.
- *
+ * 
  * Write EBML header.
  */
 void
 gst_ebml_write_header (GstEbmlWrite * ebml, const gchar * doctype,
-    guint version, guint readversion)
+    guint version)
 {
   guint64 pos;
 
@@ -929,7 +934,7 @@ gst_ebml_write_header (GstEbmlWrite * ebml, const gchar * doctype,
 #endif
   gst_ebml_write_ascii (ebml, GST_EBML_ID_DOCTYPE, doctype);
   gst_ebml_write_uint (ebml, GST_EBML_ID_DOCTYPEVERSION, version);
-  gst_ebml_write_uint (ebml, GST_EBML_ID_DOCTYPEREADVERSION, readversion);
+  gst_ebml_write_uint (ebml, GST_EBML_ID_DOCTYPEREADVERSION, version);
   gst_ebml_write_master_finish (ebml, pos);
   gst_ebml_write_flush_cache (ebml, FALSE, 0);
 }
