@@ -77,7 +77,7 @@ GstMatroskaPad;
 
 struct _GstMatroskaMux {
   GstElement     element;
-
+  
   /* < private > */
 
   /* pads */
@@ -91,9 +91,6 @@ struct _GstMatroskaMux {
   /* Application name (for the writing application header element) */
   gchar          *writing_app;
 
-  /* Date (for the DateUTC header element) */
-  GDateTime      *creation_time;
-
   /* EBML DocType. */
   const gchar    *doctype;
 
@@ -107,16 +104,13 @@ struct _GstMatroskaMux {
   GstMatroskaIndex *index;
   guint          num_indexes;
   GstClockTimeDiff min_index_interval;
-
+  gboolean       streamable;
+ 
   /* timescale in the file */
   guint64        time_scale;
-  /* minimum and maximum limit of nanoseconds you can have in a cluster */
+  /* based on timescale, limit of nanoseconds you can have in a cluster */ 
   guint64        max_cluster_duration;
-  guint64        min_cluster_duration;
 
-  /* earliest timestamp (time, ns) if offsetting to zero */
-  gboolean       offset_to_zero;
-  guint64        earliest_time;
   /* length, position (time, ns) */
   guint64        duration;
 
@@ -141,11 +135,11 @@ struct _GstMatroskaMux {
   /* GstForceKeyUnit event */
   GstEvent       *force_key_unit_event;
 
-  /* Internal Toc (adjusted UIDs and title tags removed when processed) */
-  GstToc         *internal_toc;
-
   /* Flag to ease handling of WebM specifics */
   gboolean is_webm;
+
+  /* used uids */
+  GArray *used_uids;
 };
 
 typedef struct _GstMatroskaMuxClass {
