@@ -20,7 +20,6 @@
 
 /**
  * SECTION:element-rtph261pay
- * @title: rtph261pay
  * @see_also: rtph261depay
  *
  * Payload encoded H.261 video frames into RTP packets according to RFC 4587.
@@ -36,19 +35,19 @@
  * encoder does not produce a continuous bit-stream but the decoder requires
  * it.
  *
- * ## Example launch line
+ * <refsect2>
+ * <title>Example launch line</title>
  * |[
  * gst-launch-1.0 videotestsrc ! avenc_h261 ! rtph261pay ! udpsink
  * ]| This will encode a test video and payload it. Refer to the rtph261depay
  * example to depayload and play the RTP stream.
- *
+ * </refsect2>
  */
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
 #endif
 
-#include "gstrtpelements.h"
 #include "gstrtph261pay.h"
 #include "gstrtputils.h"
 #include <gst/rtp/gstrtpbuffer.h>
@@ -83,10 +82,8 @@ static GstStaticPadTemplate gst_rtp_h261_pay_src_template =
         "clock-rate = (int) 90000, " "encoding-name = (string) \"H261\"")
     );
 
-#define parent_class gst_rtp_h261_pay_parent_class
 G_DEFINE_TYPE (GstRtpH261Pay, gst_rtp_h261_pay, GST_TYPE_RTP_BASE_PAYLOAD);
-GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (rtph261pay, "rtph261pay",
-    GST_RANK_SECONDARY, GST_TYPE_RTP_H261_PAY, rtp_element_init (plugin));
+#define parent_class gst_rtp_h261_pay_parent_class
 
 typedef struct
 {
@@ -776,7 +773,7 @@ find_gob (GstRtpH261Pay * pay, const guint8 * data, guint size, guint pos)
   return ret;
 }
 
-/* Scans after all GOB start codes and initializes the GOB structure with start
+/* Scans after all GOB start codes and initalizes the GOB structure with start
  * and end positions. */
 static ParseReturn
 gst_rtp_h261_pay_init_gobs (GstRtpH261Pay * pay, Gob * gobs, gint num_gobs,
@@ -816,9 +813,8 @@ gst_rtp_h261_pay_fragment_push (GstRtpH261Pay * pay, GstBuffer * buffer,
 
   nbytes = bitrange_to_bytes (start, end);
 
-  outbuf =
-      gst_rtp_base_payload_allocate_output_buffer (GST_RTP_BASE_PAYLOAD (pay),
-      nbytes + GST_RTP_H261_PAYLOAD_HEADER_LEN, 0, 0);
+  outbuf = gst_rtp_buffer_new_allocate (nbytes +
+      GST_RTP_H261_PAYLOAD_HEADER_LEN, 0, 0);
   gst_rtp_buffer_map (outbuf, GST_MAP_WRITE, &rtp);
   payload = gst_rtp_buffer_get_payload (&rtp);
   header = (GstRtpH261PayHeader *) payload;
@@ -844,8 +840,6 @@ gst_rtp_h261_pay_fragment_push (GstRtpH261Pay * pay, GstBuffer * buffer,
       bits + GST_ROUND_DOWN_8 (start) / 8, nbytes);
 
   GST_BUFFER_TIMESTAMP (outbuf) = pay->timestamp;
-  if (marker)
-    GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_FLAG_MARKER);
   gst_rtp_buffer_set_marker (&rtp, marker);
   pay->offset = end & 7;
 
@@ -856,7 +850,8 @@ gst_rtp_h261_pay_fragment_push (GstRtpH261Pay * pay, GstBuffer * buffer,
 
   gst_rtp_buffer_unmap (&rtp);
 
-  gst_rtp_copy_video_meta (pay, outbuf, buffer);
+  gst_rtp_copy_meta (GST_ELEMENT_CAST (pay), outbuf, buffer,
+      g_quark_from_static_string (GST_META_TAG_VIDEO_STR));
 
   return gst_rtp_base_payload_push (GST_RTP_BASE_PAYLOAD_CAST (pay), outbuf);
 }
@@ -872,7 +867,7 @@ gst_rtp_h261_packetize_and_push (GstRtpH261Pay * pay, GstBuffer * buffer,
       gst_rtp_buffer_calc_payload_len (GST_RTP_BASE_PAYLOAD_MTU (pay) -
       GST_RTP_H261_PAYLOAD_HEADER_LEN, 0, 0);
   guint startpos;
-  gint num_gobs = 0;
+  gint num_gobs;
   Gob gobs[MAX_NUM_GOB];
   Gob *gob;
   Macroblock last_mb_in_previous_packet = { 0 };
@@ -953,7 +948,7 @@ gst_rtp_h261_pay_shift_buffer (GstRtpH261Pay * pay, const guint8 * data,
     gsize size, gint offset, gsize * newsize)
 {
   /* In order to read variable length codes at the very end of the buffer
-   * without peeking into possibly unallocated data, we pad with extra 0's
+   * wihout peeking into possibly unallocated data, we pad with extra 0's
    * which will generate an invalid code at the end of the buffer. */
   guint pad = 4;
   gsize allocsize = size + pad;
@@ -1052,10 +1047,10 @@ gst_rtp_h261_pay_class_init (GstRtpH261PayClass * klass)
   element_class = GST_ELEMENT_CLASS (klass);
   gstrtpbasepayload_class = GST_RTP_BASE_PAYLOAD_CLASS (klass);
 
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_rtp_h261_pay_src_template);
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_rtp_h261_pay_sink_template);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_rtp_h261_pay_src_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_rtp_h261_pay_sink_template));
 
   gst_element_class_set_static_metadata (element_class,
       "RTP H261 packet payloader", "Codec/Payloader/Network/RTP",
@@ -1068,3 +1063,10 @@ gst_rtp_h261_pay_class_init (GstRtpH261PayClass * klass)
   GST_DEBUG_CATEGORY_INIT (rtph261pay_debug, "rtph261pay", 0,
       "H261 RTP Payloader");
 }
+
+gboolean
+gst_rtp_h261_pay_plugin_init (GstPlugin * plugin)
+{
+  return gst_element_register (plugin, "rtph261pay",
+      GST_RANK_SECONDARY, GST_TYPE_RTP_H261_PAY);
+}
