@@ -24,36 +24,26 @@
 #ifndef __TYPES_COMPAT_H__
 #define __TYPES_COMPAT_H__
 
-#ifdef   __linux__
-#include <linux/types.h>
-#include <asm/ioctl.h>
-
-#else /* One of the BSDs */
 /* From linux/types.h */
-#ifndef __bitwise__
-#  ifdef __CHECKER__
-#    define __bitwise__ __attribute__((bitwise))
-#  else
-#    define __bitwise__
-#  endif
+#ifdef __CHECKER__
+#define __bitwise__ __attribute__((bitwise))
+#else
+#define __bitwise__
+#endif
+#ifdef __CHECK_ENDIAN__
+#define __bitwise __bitwise__
+#else
+#define __bitwise
 #endif
 
-#ifndef __bitwise
-#  ifdef __CHECK_ENDIAN__
-#    define __bitwise __bitwise__
-#  else
-#    define __bitwise
-#  endif
-#endif
+#define __u64 guint64
+#define __u32 guint32
+#define __u16 guint16
+#define __u8 guint8
+#define __s64 gint64
+#define __s32 gint32
+#define __le32 guint32 __bitwise
 
-typedef guint8  __u8;
-typedef guint16 __u16;
-typedef gint32  __s32;
-typedef guint32 __u32;
-typedef gint64  __s64;
-typedef guint64 __u64;
-typedef guint32 __bitwise __le32;
-
-#endif
+#define __user
 
 #endif /* __TYPES_COMPAT_H__ */
