@@ -320,11 +320,8 @@ poll_eos (GstElement * element)
   gst_object_unref (bus);
 }
 
-/* This also polls for EOS since the TAG message comes right before the end of
- * streams. */
-
 static GstTagList *
-poll_tags (GstElement * element)
+poll_tags_only (GstElement * element)
 {
   GstBus *bus = gst_element_get_bus (element);
   GstTagList *tag_list;
@@ -337,6 +334,17 @@ poll_tags (GstElement * element)
   gst_message_unref (message);
   gst_object_unref (bus);
 
+  return tag_list;
+}
+
+/* This also polls for EOS since the TAG message comes right before the end of
+ * streams. */
+
+static GstTagList *
+poll_tags_followed_by_eos (GstElement * element)
+{
+  GstTagList *tag_list = poll_tags_only (element);
+
   poll_eos (element);
 
   return tag_list;
@@ -738,7 +746,7 @@ GST_START_TEST (test_no_buffer_album_2)
     push_buffer (test_buffer_square_float_mono (&accumulator, 44100, 512,
             0.25));
   send_eos_event (element);
-  tag_list = poll_tags (element);
+  tag_list = poll_tags_followed_by_eos (element);
   fail_unless_track_peak (tag_list, 0.25);
   fail_unless_track_gain (tag_list, -6.20);
   /* Album is not finished yet: */
@@ -752,7 +760,7 @@ GST_START_TEST (test_no_buffer_album_2)
   send_segment_event (element);
   send_eos_event (element);
 
-  tag_list = poll_tags (element);
+  tag_list = poll_tags_followed_by_eos (element);
   fail_unless_album_peak (tag_list, 0.25);
   fail_unless_album_gain (tag_list, -6.20);
   /* No track tags should be posted, as there was no data for it: */
@@ -839,7 +847,7 @@ GST_START_TEST (test_peak_float)
   send_segment_event (element);
   push_buffer (test_buffer_const_float_stereo (8000, 512, -1.369, 0.0));
   send_eos_event (element);
-  tag_list = poll_tags (element);
+  tag_list = poll_tags_followed_by_eos (element);
   fail_unless_track_peak (tag_list, 1.369);
   gst_tag_list_unref (tag_list);
 
@@ -848,7 +856,7 @@ GST_START_TEST (test_peak_float)
   send_segment_event (element);
   push_buffer (test_buffer_const_float_stereo (8000, 512, 0.0, -1.369));
   send_eos_event (element);
-  tag_list = poll_tags (element);
+  tag_list = poll_tags_followed_by_eos (element);
   fail_unless_track_peak (tag_list, 1.369);
   gst_tag_list_unref (tag_list);
 
@@ -858,7 +866,7 @@ GST_START_TEST (test_peak_float)
   send_segment_event (element);
   push_buffer (test_buffer_const_float_mono (8000, 512, -1.369));
   send_eos_event (element);
-  tag_list = poll_tags (element);
+  tag_list = poll_tags_followed_by_eos (element);
   fail_unless_track_peak (tag_list, 1.369);
   gst_tag_list_unref (tag_list);
 
@@ -880,7 +888,7 @@ GST_START_TEST (test_peak_int16_16)
   send_segment_event (element);
   push_buffer (test_buffer_const_int16_stereo (8000, 16, 512, 1 << 14, 0));
   send_eos_event (element);
-  tag_list = poll_tags (element);
+  tag_list = poll_tags_followed_by_eos (element);
   fail_unless_track_peak (tag_list, 0.5);
   gst_tag_list_unref (tag_list);
 
@@ -890,7 +898,7 @@ GST_START_TEST (test_peak_int16_16)
   send_segment_event (element);
   push_buffer (test_buffer_const_int16_stereo (8000, 16, 512, 0, 1 << 14));
   send_eos_event (element);
-  tag_list = poll_tags (element);
+  tag_list = poll_tags_followed_by_eos (element);
   fail_unless_track_peak (tag_list, 0.5);
   gst_tag_list_unref (tag_list);
 
@@ -900,7 +908,7 @@ GST_START_TEST (test_peak_int16_16)
   send_segment_event (element);
   push_buffer (test_buffer_const_int16_mono (8000, 16, 512, 1 << 14));
   send_eos_event (element);
-  tag_list = poll_tags (element);
+  tag_list = poll_tags_followed_by_eos (element);
   fail_unless_track_peak (tag_list, 0.5);
   gst_tag_list_unref (tag_list);
 
@@ -910,7 +918,7 @@ GST_START_TEST (test_peak_int16_16)
   send_segment_event (element);
   push_buffer (test_buffer_const_int16_stereo (8000, 16, 512, -1 << 14, 0));
   send_eos_event (element);
-  tag_list = poll_tags (element);
+  tag_list = poll_tags_followed_by_eos (element);
   fail_unless_track_peak (tag_list, 0.5);
   gst_tag_list_unref (tag_list);
 
@@ -920,7 +928,7 @@ GST_START_TEST (test_peak_int16_16)
   send_segment_event (element);
   push_buffer (test_buffer_const_int16_stereo (8000, 16, 512, 0, -1 << 14));
   send_eos_event (element);
-  tag_list = poll_tags (element);
+  tag_list = poll_tags_followed_by_eos (element);
   fail_unless_track_peak (tag_list, 0.5);
   gst_tag_list_unref (tag_list);
 
@@ -930,7 +938,7 @@ GST_START_TEST (test_peak_int16_16)
   send_segment_event (element);
   push_buffer (test_buffer_const_int16_mono (8000, 16, 512, -1 << 14));
   send_eos_event (element);
-  tag_list = poll_tags (element);
+  tag_list = poll_tags_followed_by_eos (element);
   fail_unless_track_peak (tag_list, 0.5);
   gst_tag_list_unref (tag_list);
 
@@ -945,7 +953,7 @@ GST_START_TEST (test_peak_int16_16)
   send_segment_event (element);
   push_buffer (test_buffer_const_int16_stereo (8000, 16, 512, 32767, 0));
   send_eos_event (element);
-  tag_list = poll_tags (element);
+  tag_list = poll_tags_followed_by_eos (element);
   fail_unless_track_peak (tag_list, 32767. / 32768.);
   gst_tag_list_unref (tag_list);
 
@@ -955,7 +963,7 @@ GST_START_TEST (test_peak_int16_16)
   send_segment_event (element);
   push_buffer (test_buffer_const_int16_stereo (8000, 16, 512, 0, 32767));
   send_eos_event (element);
-  tag_list = poll_tags (element);
+  tag_list = poll_tags_followed_by_eos (element);
   fail_unless_track_peak (tag_list, 32767. / 32768.);
   gst_tag_list_unref (tag_list);
 
@@ -965,7 +973,7 @@ GST_START_TEST (test_peak_int16_16)
   send_segment_event (element);
   push_buffer (test_buffer_const_int16_mono (8000, 16, 512, 32767));
   send_eos_event (element);
-  tag_list = poll_tags (element);
+  tag_list = poll_tags_followed_by_eos (element);
   fail_unless_track_peak (tag_list, 32767. / 32768.);
   gst_tag_list_unref (tag_list);
 
@@ -976,7 +984,7 @@ GST_START_TEST (test_peak_int16_16)
   send_segment_event (element);
   push_buffer (test_buffer_const_int16_stereo (8000, 16, 512, -32768, 0));
   send_eos_event (element);
-  tag_list = poll_tags (element);
+  tag_list = poll_tags_followed_by_eos (element);
   fail_unless_track_peak (tag_list, 1.0);
   gst_tag_list_unref (tag_list);
 
@@ -986,7 +994,7 @@ GST_START_TEST (test_peak_int16_16)
   send_segment_event (element);
   push_buffer (test_buffer_const_int16_stereo (8000, 16, 512, 0, -32768));
   send_eos_event (element);
-  tag_list = poll_tags (element);
+  tag_list = poll_tags_followed_by_eos (element);
   fail_unless_track_peak (tag_list, 1.0);
   gst_tag_list_unref (tag_list);
 
@@ -996,7 +1004,7 @@ GST_START_TEST (test_peak_int16_16)
   send_segment_event (element);
   push_buffer (test_buffer_const_int16_mono (8000, 16, 512, -32768));
   send_eos_event (element);
-  tag_list = poll_tags (element);
+  tag_list = poll_tags_followed_by_eos (element);
   fail_unless_track_peak (tag_list, 1.0);
   gst_tag_list_unref (tag_list);
 
@@ -1018,7 +1026,7 @@ GST_START_TEST (test_peak_album)
   send_segment_event (element);
   push_buffer (test_buffer_const_float_stereo (8000, 1024, 1.0, 0.0));
   send_eos_event (element);
-  tag_list = poll_tags (element);
+  tag_list = poll_tags_followed_by_eos (element);
   fail_unless_track_peak (tag_list, 1.0);
   fail_if_album_tags (tag_list);
   gst_tag_list_unref (tag_list);
@@ -1028,7 +1036,7 @@ GST_START_TEST (test_peak_album)
   send_segment_event (element);
   push_buffer (test_buffer_const_float_stereo (8000, 1024, 0.0, 0.5));
   send_eos_event (element);
-  tag_list = poll_tags (element);
+  tag_list = poll_tags_followed_by_eos (element);
   fail_unless_track_peak (tag_list, 0.5);
   fail_unless_album_peak (tag_list, 1.0);
   gst_tag_list_unref (tag_list);
@@ -1041,7 +1049,7 @@ GST_START_TEST (test_peak_album)
   send_segment_event (element);
   push_buffer (test_buffer_const_float_stereo (8000, 1024, 0.4, 0.4));
   send_eos_event (element);
-  tag_list = poll_tags (element);
+  tag_list = poll_tags_followed_by_eos (element);
   fail_unless_track_peak (tag_list, 0.4);
   fail_if_album_tags (tag_list);
   gst_tag_list_unref (tag_list);
@@ -1051,7 +1059,7 @@ GST_START_TEST (test_peak_album)
   send_segment_event (element);
   push_buffer (test_buffer_const_float_stereo (8000, 1024, 0.45, 0.45));
   send_eos_event (element);
-  tag_list = poll_tags (element);
+  tag_list = poll_tags_followed_by_eos (element);
   fail_unless_track_peak (tag_list, 0.45);
   fail_if_album_tags (tag_list);
   gst_tag_list_unref (tag_list);
@@ -1061,7 +1069,7 @@ GST_START_TEST (test_peak_album)
   send_segment_event (element);
   push_buffer (test_buffer_const_float_stereo (8000, 1024, 0.2, 0.2));
   send_eos_event (element);
-  tag_list = poll_tags (element);
+  tag_list = poll_tags_followed_by_eos (element);
   fail_unless_track_peak (tag_list, 0.2);
   fail_unless_album_peak (tag_list, 0.45);
   gst_tag_list_unref (tag_list);
@@ -1073,7 +1081,7 @@ GST_START_TEST (test_peak_album)
   send_segment_event (element);
   push_buffer (test_buffer_const_float_stereo (8000, 1024, 0.1, 0.1));
   send_eos_event (element);
-  tag_list = poll_tags (element);
+  tag_list = poll_tags_followed_by_eos (element);
   fail_unless_track_peak (tag_list, 0.1);
   fail_if_album_tags (tag_list);
   gst_tag_list_unref (tag_list);
@@ -1097,7 +1105,7 @@ GST_START_TEST (test_peak_track_album)
   send_segment_event (element);
   push_buffer (test_buffer_const_float_mono (8000, 1024, 1.0));
   send_eos_event (element);
-  tag_list = poll_tags (element);
+  tag_list = poll_tags_followed_by_eos (element);
   fail_unless_track_peak (tag_list, 1.0);
   fail_if_album_tags (tag_list);
   gst_tag_list_unref (tag_list);
@@ -1107,7 +1115,7 @@ GST_START_TEST (test_peak_track_album)
   send_segment_event (element);
   push_buffer (test_buffer_const_float_mono (8000, 1024, 0.5));
   send_eos_event (element);
-  tag_list = poll_tags (element);
+  tag_list = poll_tags_followed_by_eos (element);
   fail_unless_track_peak (tag_list, 0.5);
   fail_unless_album_peak (tag_list, 0.5);
   gst_tag_list_unref (tag_list);
@@ -1136,7 +1144,7 @@ GST_START_TEST (test_peak_album_abort_to_track)
   send_segment_event (element);
   push_buffer (test_buffer_const_float_stereo (8000, 1024, 1.0, 0.0));
   send_eos_event (element);
-  tag_list = poll_tags (element);
+  tag_list = poll_tags_followed_by_eos (element);
   fail_unless_track_peak (tag_list, 1.0);
   fail_if_album_tags (tag_list);
   gst_tag_list_unref (tag_list);
@@ -1148,7 +1156,7 @@ GST_START_TEST (test_peak_album_abort_to_track)
   send_segment_event (element);
   push_buffer (test_buffer_const_float_stereo (8000, 1024, 0.0, 0.5));
   send_eos_event (element);
-  tag_list = poll_tags (element);
+  tag_list = poll_tags_followed_by_eos (element);
   fail_unless_track_peak (tag_list, 0.5);
   fail_if_album_tags (tag_list);
   gst_tag_list_unref (tag_list);
@@ -1178,7 +1186,7 @@ GST_START_TEST (test_gain_album)
     push_buffer (test_buffer_square_float_stereo (&accumulator, 44100, 512,
             0.75, 0.75));
   send_eos_event (element);
-  tag_list = poll_tags (element);
+  tag_list = poll_tags_followed_by_eos (element);
   fail_unless_track_peak (tag_list, 0.75);
   fail_unless_track_gain (tag_list, -15.70);
   fail_if_album_tags (tag_list);
@@ -1191,7 +1199,7 @@ GST_START_TEST (test_gain_album)
     push_buffer (test_buffer_square_float_stereo (&accumulator, 44100, 512,
             0.5, 0.5));
   send_eos_event (element);
-  tag_list = poll_tags (element);
+  tag_list = poll_tags_followed_by_eos (element);
   fail_unless_track_peak (tag_list, 0.5);
   fail_unless_track_gain (tag_list, -12.22);
   fail_if_album_tags (tag_list);
@@ -1205,7 +1213,7 @@ GST_START_TEST (test_gain_album)
             0.25, 0.25));
   send_eos_event (element);
 
-  tag_list = poll_tags (element);
+  tag_list = poll_tags_followed_by_eos (element);
   fail_unless_track_peak (tag_list, 0.25);
   fail_unless_track_gain (tag_list, -6.20);
   fail_unless_album_peak (tag_list, 0.75);
@@ -1243,8 +1251,13 @@ GST_START_TEST (test_forced)
   for (i = 20; i--;)
     push_buffer (test_buffer_const_float_stereo (44100, 512, 0.5, 0.5));
   send_eos_event (element);
+
   /* This fails if a tag message is generated: */
-  poll_eos (element);
+  /* Same values as above */
+  tag_list = poll_tags_followed_by_eos (element);
+  fail_unless_track_gain (tag_list, 2.21);
+  fail_unless_track_peak (tag_list, 1.0);
+  gst_tag_list_unref (tag_list);
 
   /* Now back to a track without tags. */
   send_flush_events (element);
@@ -1253,7 +1266,7 @@ GST_START_TEST (test_forced)
     push_buffer (test_buffer_square_float_stereo (&accumulator, 44100, 512,
             0.25, 0.25));
   send_eos_event (element);
-  tag_list = poll_tags (element);
+  tag_list = poll_tags_followed_by_eos (element);
   fail_unless_track_peak (tag_list, 0.25);
   fail_unless_track_gain (tag_list, get_expected_gain (44100));
   gst_tag_list_unref (tag_list);
@@ -1292,8 +1305,15 @@ GST_START_TEST (test_forced_separate)
     push_buffer (test_buffer_square_float_stereo (&accumulator, 44100, 512,
             0.5, 0.5));
   send_eos_event (element);
-  /* This fails if a tag message is generated: */
-  poll_eos (element);
+
+  /* Same values as above */
+  tag_list = poll_tags_only (element);
+  fail_unless_track_gain (tag_list, 2.21);
+  gst_tag_list_unref (tag_list);
+
+  tag_list = poll_tags_followed_by_eos (element);
+  fail_unless_track_peak (tag_list, 1.0);
+  gst_tag_list_unref (tag_list);
 
   /* Now a track without tags. */
   send_flush_events (element);
@@ -1303,7 +1323,7 @@ GST_START_TEST (test_forced_separate)
     push_buffer (test_buffer_square_float_stereo (&accumulator, 44100, 512,
             0.25, 0.25));
   send_eos_event (element);
-  tag_list = poll_tags (element);
+  tag_list = poll_tags_followed_by_eos (element);
   fail_unless_track_peak (tag_list, 0.25);
   fail_unless_track_gain (tag_list, get_expected_gain (44100));
   fail_if_album_tags (tag_list);
@@ -1342,7 +1362,12 @@ GST_START_TEST (test_forced_after_data)
   send_tag_event (element, tag_list);
 
   send_eos_event (element);
-  poll_eos (element);
+
+  /* Same values as above */
+  tag_list = poll_tags_followed_by_eos (element);
+  fail_unless_track_gain (tag_list, 2.21);
+  fail_unless_track_peak (tag_list, 1.0);
+  gst_tag_list_unref (tag_list);
 
   send_flush_events (element);
   send_segment_event (element);
@@ -1351,7 +1376,7 @@ GST_START_TEST (test_forced_after_data)
     push_buffer (test_buffer_square_float_stereo (&accumulator, 8000, 512, 0.25,
             0.25));
   send_eos_event (element);
-  tag_list = poll_tags (element);
+  tag_list = poll_tags_followed_by_eos (element);
   fail_unless_track_peak (tag_list, 0.25);
   fail_unless_track_gain (tag_list, get_expected_gain (8000));
   gst_tag_list_unref (tag_list);
@@ -1388,8 +1413,12 @@ GST_START_TEST (test_forced_album)
     push_buffer (test_buffer_square_float_stereo (&accumulator, 44100, 512,
             0.5, 0.5));
   send_eos_event (element);
-  /* This fails if a tag message is generated: */
-  poll_eos (element);
+
+  /* Same values as above */
+  tag_list = poll_tags_followed_by_eos (element);
+  fail_unless_track_gain (tag_list, 2.21);
+  fail_unless_track_peak (tag_list, 1.0);
+  gst_tag_list_unref (tag_list);
 
   /* Now an album without tags. */
   g_object_set (element, "num-tracks", 2, NULL);
@@ -1401,7 +1430,7 @@ GST_START_TEST (test_forced_album)
     push_buffer (test_buffer_square_float_stereo (&accumulator, 44100, 512,
             0.25, 0.25));
   send_eos_event (element);
-  tag_list = poll_tags (element);
+  tag_list = poll_tags_followed_by_eos (element);
   fail_unless_track_peak (tag_list, 0.25);
   fail_unless_track_gain (tag_list, get_expected_gain (44100));
   fail_if_album_tags (tag_list);
@@ -1415,7 +1444,7 @@ GST_START_TEST (test_forced_album)
     push_buffer (test_buffer_square_float_stereo (&accumulator, 44100, 512,
             0.25, 0.25));
   send_eos_event (element);
-  tag_list = poll_tags (element);
+  tag_list = poll_tags_followed_by_eos (element);
   fail_unless_track_peak (tag_list, 0.25);
   fail_unless_track_gain (tag_list, get_expected_gain (44100));
   fail_unless_album_peak (tag_list, 0.25);
@@ -1452,7 +1481,13 @@ GST_START_TEST (test_forced_album_skip)
     push_buffer (test_buffer_square_float_stereo (&accumulator, 8000, 512, 0.25,
             0.25));
   send_eos_event (element);
-  poll_eos (element);
+
+  /* Same values as above */
+  tag_list = poll_tags_followed_by_eos (element);
+  fail_unless_track_gain (tag_list, 2.21);
+  fail_unless_track_peak (tag_list, 0.75);
+  gst_tag_list_unref (tag_list);
+
   fail_unless_num_tracks (element, 1);
 
   /* This track has no tags, but needs to be skipped anyways since we
@@ -1473,7 +1508,7 @@ GST_START_TEST (test_forced_album_skip)
     push_buffer (test_buffer_square_float_stereo (&accumulator, 8000, 512, 0.25,
             0.25));
   send_eos_event (element);
-  tag_list = poll_tags (element);
+  tag_list = poll_tags_followed_by_eos (element);
   fail_unless_track_peak (tag_list, 0.25);
   fail_unless_track_gain (tag_list, get_expected_gain (8000));
   fail_if_album_tags (tag_list);
@@ -1501,7 +1536,7 @@ GST_START_TEST (test_forced_album_no_skip)
     push_buffer (test_buffer_square_float_stereo (&accumulator, 8000, 512, 0.25,
             0.25));
   send_eos_event (element);
-  tag_list = poll_tags (element);
+  tag_list = poll_tags_followed_by_eos (element);
   fail_unless_track_peak (tag_list, 0.25);
   fail_unless_track_gain (tag_list, get_expected_gain (8000));
   fail_if_album_tags (tag_list);
@@ -1521,7 +1556,15 @@ GST_START_TEST (test_forced_album_no_skip)
   for (i = 20; i--;)
     push_buffer (test_buffer_const_float_stereo (8000, 512, 0.0, 0.0));
   send_eos_event (element);
-  tag_list = poll_tags (element);
+
+  /* the first batch from the tags */
+  tag_list = poll_tags_only (element);
+  fail_unless_track_peak (tag_list, 0.75);
+  fail_unless_track_gain (tag_list, 2.21);
+  gst_tag_list_unref (tag_list);
+
+  /* the second from the processing */
+  tag_list = poll_tags_followed_by_eos (element);
   fail_unless_track_peak (tag_list, 0.0);
   fail_unless_track_gain (tag_list, SILENCE_GAIN);
   /* Second track was just silence so the album peak equals the first
@@ -1555,7 +1598,7 @@ GST_START_TEST (test_forced_abort_album_no_skip)
     push_buffer (test_buffer_square_float_stereo (&accumulator, 8000, 512, 0.25,
             0.25));
   send_eos_event (element);
-  tag_list = poll_tags (element);
+  tag_list = poll_tags_followed_by_eos (element);
   fail_unless_track_peak (tag_list, 0.25);
   fail_unless_track_gain (tag_list, get_expected_gain (8000));
   fail_if_album_tags (tag_list);
@@ -1578,7 +1621,11 @@ GST_START_TEST (test_forced_abort_album_no_skip)
   for (i = 20; i--;)
     push_buffer (test_buffer_const_float_stereo (8000, 512, 0.0, 0.0));
   send_eos_event (element);
-  poll_eos (element);
+
+  tag_list = poll_tags_followed_by_eos (element);
+  fail_unless_track_peak (tag_list, 0.75);
+  fail_unless_track_gain (tag_list, 2.21);
+  gst_tag_list_unref (tag_list);
 
   cleanup_rganalysis (element);
 }
@@ -1602,7 +1649,7 @@ GST_START_TEST (test_reference_level)
     push_buffer (test_buffer_square_float_stereo (&accumulator, 44100, 512,
             0.25, 0.25));
   send_eos_event (element);
-  tag_list = poll_tags (element);
+  tag_list = poll_tags_followed_by_eos (element);
   fail_unless_track_peak (tag_list, 0.25);
   fail_unless_track_gain (tag_list, get_expected_gain (44100));
   fail_if_album_tags (tag_list);
@@ -1619,7 +1666,7 @@ GST_START_TEST (test_reference_level)
     push_buffer (test_buffer_square_float_stereo (&accumulator, 44100, 512,
             0.25, 0.25));
   send_eos_event (element);
-  tag_list = poll_tags (element);
+  tag_list = poll_tags_followed_by_eos (element);
   fail_unless_track_peak (tag_list, 0.25);
   fail_unless_track_gain (tag_list, get_expected_gain (44100) - 6.);
   fail_if_album_tags (tag_list);
@@ -1635,7 +1682,7 @@ GST_START_TEST (test_reference_level)
     push_buffer (test_buffer_square_float_stereo (&accumulator, 44100, 512,
             0.25, 0.25));
   send_eos_event (element);
-  tag_list = poll_tags (element);
+  tag_list = poll_tags_followed_by_eos (element);
   fail_unless_track_peak (tag_list, 0.25);
   fail_unless_track_gain (tag_list, get_expected_gain (44100) - 6.);
   fail_unless_album_peak (tag_list, 0.25);
@@ -1682,7 +1729,7 @@ GST_START_TEST (test_all_formats)
       push_buffer (test_buffer_square_int16_mono (&accumulator,
               supported_rates[i].sample_rate, 16, 512, 1 << 13));
     send_eos_event (element);
-    tag_list = poll_tags (element);
+    tag_list = poll_tags_followed_by_eos (element);
     fail_unless_track_peak (tag_list, 0.25);
     fail_unless_track_gain (tag_list, supported_rates[i].gain);
     gst_tag_list_unref (tag_list);
@@ -1715,7 +1762,7 @@ GST_END_TEST;
     push_buffer (test_buffer_square_float_mono (&accumulator,         \
             sample_rate, 512, 0.25));                                 \
   send_eos_event (element);                                           \
-  tag_list = poll_tags (element);                                     \
+  tag_list = poll_tags_followed_by_eos (element);                                     \
   fail_unless_track_peak (tag_list, 0.25);                            \
   fail_unless_track_gain (tag_list,                                   \
       get_expected_gain (sample_rate));                               \
@@ -1743,7 +1790,7 @@ GST_END_TEST;
     push_buffer (test_buffer_square_float_stereo (&accumulator,       \
             sample_rate, 512, 0.25, 0.25));                           \
   send_eos_event (element);                                           \
-  tag_list = poll_tags (element);                                     \
+  tag_list = poll_tags_followed_by_eos (element);                                     \
   fail_unless_track_peak (tag_list, 0.25);                            \
   fail_unless_track_gain (tag_list,                                   \
       get_expected_gain (sample_rate));                               \
@@ -1772,7 +1819,7 @@ GST_END_TEST;
             sample_rate, depth, 512, 1 << (13 + depth - 16)));        \
                                                                       \
   send_eos_event (element);                                           \
-  tag_list = poll_tags (element);                                     \
+  tag_list = poll_tags_followed_by_eos (element);                                     \
   fail_unless_track_peak (tag_list, 0.25);                            \
   fail_unless_track_gain (tag_list,                                   \
       get_expected_gain (sample_rate));                               \
@@ -1801,7 +1848,7 @@ GST_END_TEST;
             sample_rate, depth, 512, 1 << (13 + depth - 16),          \
             1 << (13 + depth - 16)));                                 \
   send_eos_event (element);                                           \
-  tag_list = poll_tags (element);                                     \
+  tag_list = poll_tags_followed_by_eos (element);                                     \
   fail_unless_track_peak (tag_list, 0.25);                            \
   fail_unless_track_gain (tag_list,                                   \
       get_expected_gain (sample_rate));                               \
@@ -1874,13 +1921,13 @@ rganalysis_suite (void)
 
   tcase_add_test (tc_chain, test_gain_album);
 
-  tcase_skip_broken_test (tc_chain, test_forced);
-  tcase_skip_broken_test (tc_chain, test_forced_separate);
-  tcase_skip_broken_test (tc_chain, test_forced_after_data);
-  tcase_skip_broken_test (tc_chain, test_forced_album);
-  tcase_skip_broken_test (tc_chain, test_forced_album_skip);
-  tcase_skip_broken_test (tc_chain, test_forced_album_no_skip);
-  tcase_skip_broken_test (tc_chain, test_forced_abort_album_no_skip);
+  tcase_add_test (tc_chain, test_forced);
+  tcase_add_test (tc_chain, test_forced_separate);
+  tcase_add_test (tc_chain, test_forced_after_data);
+  tcase_add_test (tc_chain, test_forced_album);
+  tcase_add_test (tc_chain, test_forced_album_skip);
+  tcase_add_test (tc_chain, test_forced_album_no_skip);
+  tcase_add_test (tc_chain, test_forced_abort_album_no_skip);
 
   tcase_add_test (tc_chain, test_reference_level);
 
@@ -1929,19 +1976,4 @@ rganalysis_suite (void)
   return s;
 }
 
-int
-main (int argc, char **argv)
-{
-  gint nf;
-
-  Suite *s = rganalysis_suite ();
-  SRunner *sr = srunner_create (s);
-
-  gst_check_init (&argc, &argv);
-
-  srunner_run_all (sr, CK_ENV);
-  nf = srunner_ntests_failed (sr);
-  srunner_free (sr);
-
-  return nf;
-}
+GST_CHECK_MAIN (rganalysis);
